@@ -3,6 +3,7 @@ import textwrap
 import ctypes
 import os
 import time
+import traceback
 
 import windows
 import windows.debug
@@ -26,7 +27,7 @@ else:
 
 yolo = generate_pop_and_exit_fixtures([pop_proc_32, pop_proc_64], ids=["proc32dbg", "proc64dbg"], dwCreationFlags=gdef.CREATE_SUSPENDED)
 
-DEFAULT_DEBUGGER_TIMEOUT = 10
+DEFAULT_DEBUGGER_TIMEOUT = 60
 
 @pytest.mark.timeout(DEFAULT_DEBUGGER_TIMEOUT)
 def test_init_breakpoint_callback(proc32_64_debug):
@@ -366,6 +367,54 @@ def test_memory_breakpoint_exec(proc32_64_debug):
     for i in range(NB_NOP_IN_PAGE + 1):
         assert TSTBP.DATA[i] == addr + i
 
+def test_memory_breakpoint_trigger_multipage(proc32_64_debug):
+    """Check that a memory breakpoint triggering on multiple page on the same instruction restore are correctly restored on all pages"""
+
+    class MultiPageMemBP(windows.debug.MemoryBreakpoint):
+        ALL_TRIGGER_ADDR = []
+
+        def trigger(self, dbg, exc):
+            pc_fault_addr = dbg.current_thread.context.pc
+            self.ALL_TRIGGER_ADDR.append(pc_fault_addr)
+            print(hex(pc_fault_addr))
+            # Stop when we have a breakpoint in the 2nd page of the alloc
+            if shellcodeaddr + 0x1000 <= pc_fault_addr <=  shellcodeaddr + 0x2000:
+                dbg.current_process.exit()
+
+    # Trigger a write that will write on both page at once,
+    # Triggering both pages mem-bp on the same instruction.
+    # Then call an instruction at the end of page to see if both page of mem-bp still trigger the BP
+
+    shellcodeaddr = proc32_64_debug.virtual_alloc(0x2000)
+
+    if proc32_64_debug.bitness == 64:
+        shellcode = x64.MultipleInstr()
+        shellcode += x64.Mov("RAX", shellcodeaddr + 0xffe)
+        shellcode += x64.Mov("RCX", 0xc3909090) # Nopnopnopret
+        shellcode += x64.Mov(x64.mem("[RAX]"), "ECX") # Will write on both page at once
+        shellcode += x64.Push("RAX")
+        shellcode += x64.Ret() # Jump on the nop + ret
+    else:
+        shellcode = x86.MultipleInstr()
+        shellcode += x86.Mov("EAX", shellcodeaddr + 0xffe)
+        shellcode += x86.Mov("ECX", 0xc3909090) # Nopnopnopret
+        shellcode += x86.Mov(x86.mem("[EAX]"), "ECX") # Will write on both page at once
+        shellcode += x86.Push("EAX")
+        shellcode += x86.Ret() # Jump on the nop + ret
+
+    d = windows.debug.Debugger(proc32_64_debug)
+    bp = MultiPageMemBP(addr=shellcodeaddr, size=0x2000, events="XW")
+    proc32_64_debug.write_memory(shellcodeaddr, shellcode.get_code())
+    d.add_bp(bp)
+    proc32_64_debug.create_thread(shellcodeaddr, 0)
+
+    d.loop()
+    # Check that the 2 nop at the end of the first page of the membp trigger
+    # If access right where not correctly restored: this would not trigger on the first page
+    assert shellcodeaddr + 0xffe in bp.ALL_TRIGGER_ADDR
+    assert shellcodeaddr + 0xfff in bp.ALL_TRIGGER_ADDR
+    assert shellcodeaddr + 0x1000 in bp.ALL_TRIGGER_ADDR
+
 
 # breakpoint remove
 import threading
@@ -375,22 +424,28 @@ import threading
 @pytest.mark.parametrize("bptype", [windows.debug.FunctionParamDumpHXBP, windows.debug.FunctionParamDumpBP])
 def test_standard_breakpoint_self_remove(proc32_64_debug, bptype):
     data = set()
-
+    thread_exception = []
     def do_check():
-        time.sleep(1)
-        print("[==================] LOADING PYTHON")
-        proc32_64_debug.execute_python_unsafe("1").wait()
-        print("[==================] OPEN SELF_FILENAME1")
-        proc32_64_debug.execute_python_unsafe("open(u'SELF_FILENAME1')").wait()
-        time.sleep(0.1)
-        print("[==================] OPEN SELF_FILENAME2")
-        proc32_64_debug.execute_python_unsafe("open(u'SELF_FILENAME2')").wait()
-        time.sleep(0.1)
-        print("[==================] OPEN SELF_FILENAME3")
-        proc32_64_debug.execute_python_unsafe("open(u'SELF_FILENAME3')").wait()
-        time.sleep(0.1)
-        print("[==================] KILLING TARGET")
-        proc32_64_debug.exit()
+        time.sleep(2)
+        try:
+            assert proc32_64_debug.peb.Ldr.contents.Initialized, "peb.Ldr not yet Initialized"
+            print("[==================] LOADING PYTHON")
+            proc32_64_debug.execute_python_unsafe("1").wait()
+            print("[==================] OPEN SELF_FILENAME1")
+            proc32_64_debug.execute_python_unsafe("open(u'SELF_FILENAME1')").wait()
+            time.sleep(0.1)
+            print("[==================] OPEN SELF_FILENAME2")
+            proc32_64_debug.execute_python_unsafe("open(u'SELF_FILENAME2')").wait()
+            time.sleep(0.1)
+            print("[==================] OPEN SELF_FILENAME3")
+            proc32_64_debug.execute_python_unsafe("open(u'SELF_FILENAME3')").wait()
+            time.sleep(0.1)
+            print("[==================] KILLING TARGET")
+        except Exception as e:
+            traceback.print_exc()
+            thread_exception.append(e)
+        finally:
+            proc32_64_debug.exit()
 
     class TSTBP(bptype):
         TARGET = windows.winproxy.CreateFileW
@@ -406,8 +461,13 @@ def test_standard_breakpoint_self_remove(proc32_64_debug, bptype):
 
     d = windows.debug.Debugger(proc32_64_debug)
     d.add_bp(TSTBP("kernelbase!CreateFileW"))
-    threading.Thread(target=do_check).start()
+    t = threading.Thread(target=do_check)
+    t.start()
     d.loop()
+    assert not t.is_alive()
+    if thread_exception:
+        raise thread_exception[0]
+
     assert data >= set([u"SELF_FILENAME1", u"SELF_FILENAME2"])
     assert u"SELF_FILENAME3" not in data
 
@@ -426,24 +486,35 @@ class MyMetaDbgDebuger(windows.debug.Debugger):
 @pytest.mark.parametrize("bptype", [windows.debug.FunctionParamDumpHXBP, windows.debug.FunctionParamDumpBP])
 def test_standard_breakpoint_remove(proc32_64_debug, bptype):
     data = set()
-
+    thread_exception = []
     def do_check():
-        time.sleep(1)
-        print("[==================] LOADING PYTHON")
-        proc32_64_debug.execute_python_unsafe("1").wait()
-        print("[==================] OPEN FILENAME1")
-        proc32_64_debug.execute_python_unsafe("open(u'FILENAME1')").wait()
-        time.sleep(0.1)
-        print("[==================] OPEN FILENAME2")
-        proc32_64_debug.execute_python_unsafe("open(u'FILENAME2')").wait()
-        time.sleep(0.1)
-        print("[==================] RM BP")
-        d.del_bp(the_bp)
-        print("[==================] OPEN FILENAME3")
-        proc32_64_debug.execute_python_unsafe("open(u'FILENAME3')").wait()
-        time.sleep(0.1)
-        print("[==================] KILLING TARGET")
-        proc32_64_debug.exit()
+        time.sleep(2)
+        try:
+            assert proc32_64_debug.peb.Ldr.contents.Initialized, "peb.Ldr not yet Initialized"
+            print("[==================] LOADING PYTHON")
+            assert list(d.breakpoints.values())[0]
+            proc32_64_debug.execute_python_unsafe("1").wait()
+            print("[==================] OPEN FILENAME1")
+            assert list(d.breakpoints.values())[0]
+            proc32_64_debug.execute_python_unsafe("open(u'FILENAME1')").wait()
+            time.sleep(0.1)
+            print("[==================] OPEN FILENAME2")
+            assert list(d.breakpoints.values())[0]
+            proc32_64_debug.execute_python_unsafe("open(u'FILENAME2')").wait()
+            time.sleep(0.1)
+            print("[==================] RM BP")
+            assert list(d.breakpoints.values())[0]
+            d.del_bp(the_bp)
+            assert not list(d.breakpoints.values())[0]
+            print("[==================] OPEN FILENAME3")
+            proc32_64_debug.execute_python_unsafe("open(u'FILENAME3')").wait()
+            time.sleep(0.1)
+            print("[==================] KILLING TARGET")
+        except Exception as e:
+            traceback.print_exc()
+            thread_exception.append(e)
+        finally:
+            proc32_64_debug.exit()
 
     class TSTBP(bptype):
         TARGET = windows.winproxy.CreateFileW
@@ -460,8 +531,12 @@ def test_standard_breakpoint_remove(proc32_64_debug, bptype):
     # import pdb;pdb.set_trace()
     d.add_bp(the_bp)
     time.sleep(0.1)
-    threading.Thread(target=do_check).start()
+    t = threading.Thread(target=do_check)
+    t.start()
     d.loop()
+    assert not t.is_alive()
+    if thread_exception:
+        raise thread_exception[0]
     assert data >= set([u"FILENAME1", u"FILENAME2"])
     assert u"FILENAME3" not in data
 
@@ -673,4 +748,6 @@ def test_keyboardinterrupt_when_bp_event(proc32_64_debug, monkeypatch):
         assert proc32_64_debug.read_memory(addr, len(TEST_CODE)) == TEST_CODE
         assert bad_thread.context.pc == addr
     else:
-        raise ValueError("Should have raised")
\ No newline at end of file
+        raise ValueError("Should have raised")
+
+
