@@ -4,9 +4,11 @@ import pytest
 import os
 import sys
 import time
-import struct
+import ctypes
+
 import textwrap
 import shutil
+import re
 
 import windows
 import windows.pipe
@@ -23,11 +25,13 @@ class TestCurrentProcessWithCheckGarbage(object):
         assert myself.ppid == windows.current_process.ppid
 
     def test_get_current_process_peb(self):
-        return windows.current_process.peb
+        assert windows.current_process.peb
 
     def test_get_current_process_modules(self):
-        # Use sys.executable because executable can be a PyInstaller exe
-        assert os.path.basename(sys.executable) in windows.current_process.peb.modules[0].name
+        # Use module filename because this executable can be:
+        # 1. A PyInstaller exe
+        # 2. A Windows App execution alias (Microsoft Store builds)
+        assert os.path.basename(windows.current_process.peb.ProcessParameters[0].ImagePathName.str) in windows.current_process.peb.modules[0].name
 
     def test_get_current_process_exe(self):
         exe = windows.current_process.peb.exe
@@ -36,12 +40,12 @@ class TestCurrentProcessWithCheckGarbage(object):
         exe.bitness ==  exe_by_module.bitness
 
     def test_current_process_pe_imports(self):
-        python_module = windows.current_process.peb.modules[0]
-        imp = python_module.pe.imports
-        assert "kernel32.dll" in imp.keys(), 'Kernel32.dll not in python imports'
-        current_proc_id_iat = [f for f in imp["kernel32.dll"] if f.name == "GetCurrentProcessId"][0]
-        k32_base = windows.winproxy.LoadLibraryA(b"kernel32.dll")
-        assert windows.winproxy.GetProcAddress(k32_base, b"GetCurrentProcessId") == current_proc_id_iat.value
+        k32_mod = windows.current_process.peb.modules[2]
+        imp = k32_mod.pe.imports
+        assert "ntdll.dll" in imp.keys(), 'ntdll.dll not in python imports'
+        fn_id_iat = [f for f in imp["ntdll.dll"] if f.name == "NtCreateFile"][0]
+        ntdll_base = windows.winproxy.LoadLibraryA(b"ntdll.dll")
+        assert windows.winproxy.GetProcAddress(ntdll_base, b"NtCreateFile") == fn_id_iat.value
 
     def test_current_process_pe_exports(self):
         mods = [m for m in windows.current_process.peb.modules if m.name == "kernel32.dll"]
@@ -166,6 +170,40 @@ class TestProcessWithCheckGarbage(object):
                 current_proc.read_memory(waddr, 0x20) # Check that Reading dumbly fails
             assert current_proc.read_wstring(waddr) ==  test_string
 
+    def test_virtual_alloc_current_process(self):
+        current_proc = windows.current_process
+        addr1 = current_proc.virtual_alloc(0x1000, prot=gdef.PAGE_EXECUTE_READWRITE)
+        meminfo = current_proc.query_memory(addr1)
+        assert meminfo.AllocationBase == addr1
+        assert meminfo.AllocationProtect == gdef.PAGE_EXECUTE_READWRITE
+        assert meminfo.Type == gdef.MEM_PRIVATE
+        assert meminfo.State == gdef.MEM_COMMIT
+        assert meminfo.Protect == gdef.PAGE_EXECUTE_READWRITE
+        del meminfo
+
+        addr2 = current_proc.virtual_alloc(0x1000, prot=gdef.PAGE_READONLY, type=gdef.MEM_RESERVE)
+        meminfo = current_proc.query_memory(addr2)
+        assert meminfo.AllocationBase == addr2
+        assert meminfo.AllocationProtect == gdef.PAGE_READONLY
+        assert meminfo.Type == gdef.MEM_PRIVATE
+        assert meminfo.State == gdef.MEM_RESERVE
+
+    def test_virtual_alloc(self, proc32_64):
+        addr1 = proc32_64.virtual_alloc(0x1000, prot=gdef.PAGE_EXECUTE_READWRITE)
+        meminfo = proc32_64.query_memory(addr1)
+        assert meminfo.AllocationBase == addr1
+        assert meminfo.AllocationProtect == gdef.PAGE_EXECUTE_READWRITE
+        assert meminfo.Type == gdef.MEM_PRIVATE
+        assert meminfo.State == gdef.MEM_COMMIT
+        assert meminfo.Protect == gdef.PAGE_EXECUTE_READWRITE
+
+        addr2 = proc32_64.virtual_alloc(0x1000, prot=gdef.PAGE_READONLY, type=gdef.MEM_RESERVE)
+        meminfo = proc32_64.query_memory(addr2)
+        assert meminfo.AllocationBase == addr2
+        assert meminfo.AllocationProtect == gdef.PAGE_READONLY
+        assert meminfo.Type == gdef.MEM_PRIVATE
+        assert meminfo.State == gdef.MEM_RESERVE
+
     def test_query_memory(self, proc32_64):
         addr = proc32_64.virtual_alloc(0x2000, prot=gdef.PAGE_EXECUTE_READWRITE)
         proc32_64.virtual_protect(addr, 0x2000, gdef.PAGE_READONLY)
@@ -191,6 +229,7 @@ class TestProcessWithCheckGarbage(object):
 
     @windows_64bit_only
     def test_execute_to_64(self, proc64):
+        assert proc64.architecture == gdef.IMAGE_FILE_MACHINE_AMD64, "TODO: better machine fixture for ARM64"
         with proc64.allocated_memory(0x1000) as addr:
             shellcode = x64.MultipleInstr()
             shellcode += x64.Mov('RAX', 0x4242424243434343)
@@ -209,6 +248,7 @@ class TestProcessWithCheckGarbage(object):
             dword = proc32_64.read_dword(addr)
             assert dword == 0x42424242
 
+    @python_injection
     def test_execute_python_good_version(self, proc32_64):
         PIPE_NAME = "PFW_TEST_Pipe"
         rcode = r"""import sys; import windows; import windows.pipe; windows.pipe.send_object("{pipe}", list(sys.version_info))"""
@@ -288,6 +328,7 @@ class TestProcessWithCheckGarbage(object):
         # Check the RemotePythonError contains the remote exception text
         assert b"ValueError: EXCEPTION_MESSAGE" in ar.value.args[0]
 
+    @python_injection
     def test_execute_python_create_console(self, proc32_64):
         res = proc32_64.execute_python("import windows; windows.utils.create_console()")
 
@@ -347,6 +388,7 @@ class TestProcessWithCheckGarbage(object):
 
     @windows_64bit_only
     def test_set_thread_context_64(self, proc64):
+        assert proc64.architecture == gdef.IMAGE_FILE_MACHINE_AMD64, "TODO: better machine fixture for ARM64"
         code =  x64.MultipleInstr()
         code += x64.Label(":LOOP")
         code += x64.Jmp(":LOOP")
@@ -364,12 +406,19 @@ class TestProcessWithCheckGarbage(object):
         time.sleep(0.1)
         assert t.exit_code == 0x11223344
 
-
+    @dll_injection
     def test_load_library(self, proc32_64):
         DLL = "wintrust.dll"
         proc32_64.load_library(DLL)
         assert DLL in [m.name for m in proc32_64.peb.modules]
 
+    @dll_injection
+    def test_load_library_suspended(self, proc32_64_suspended):
+        DLL = "wintrust.dll"
+        proc32_64_suspended.load_library(DLL)
+        assert DLL in [m.name for m in proc32_64_suspended.peb.modules]
+
+    @dll_injection
     def test_load_library_unicode_name(self, proc32_64, tmpdir):
         mybitness = windows.current_process.bitness
         UNICODE_FILENAME = u'\u4e2d\u56fd\u94f6\u884c\u7f51\u94f6\u52a9\u624b.dll'
@@ -465,19 +514,48 @@ class TestProcessWithCheckGarbage(object):
         with proc32_64.allocated_memory(0x1000) as addr:
             assert proc32_64.get_mapped_filename(addr) is None
 
+    def test_current_thread_teb(self):
+        teb = windows.current_thread.teb
+        assert ctypes.addressof(teb) == ctypes.addressof(windows.current_thread.teb.NtTib.Self[0])
+        assert ctypes.addressof(windows.current_process.peb) == ctypes.addressof(teb.ProcessEnvironmentBlock[0])
+        # Check type of teb.peb is the correct subclass (with modules & co)
+        assert teb.peb.modules
 
+    @cross_heaven_gates
     def test_thread_teb_base(self, proc32_64):
         t = proc32_64.threads[0]
         assert t.teb_base != 0
 
+    @cross_heaven_gates
+    def test_teb(self, proc32_64):
+        teb = proc32_64.threads[0].teb
+        if proc32_64.bitness == 32:
+            assert type(teb) == windows.winobject.process.RemoteTEB32
+        else:
+            assert type(teb) == windows.winobject.process.RemoteTEB64
+        assert teb.NtTib.Self.value == teb._base_addr
+        assert teb.ProcessEnvironmentBlock.value == teb.peb._base_addr
+        # Check type of teb.peb is the correct subclass (with modules & co)
+        assert teb.peb.modules
+
     @windows_64bit_only
+    @cross_heaven_gates
     def test_thread_teb_syswow_base(self, proc32):
         t = proc32.threads[0]
         assert t.teb_base != 0
         assert t.teb_syswow_base != 0
         assert t.teb_base == t.teb_syswow_base + 0x2000
 
-
+    @windows_64bit_only
+    @cross_heaven_gates
+    def test_thread_teb_syswow(self, proc32):
+        teb_syswow = proc32.threads[0].teb_syswow
+        assert type(teb_syswow) == windows.winobject.process.RemoteTEB64
+        assert type(teb_syswow.peb) == windows.winobject.process.RemotePEB64
+        assert teb_syswow.NtTib.Self.value == teb_syswow._base_addr
+        assert teb_syswow.ProcessEnvironmentBlock.value == teb_syswow.peb._base_addr
+        # Check type of teb.peb is the correct subclass (with modules & co)
+        assert teb_syswow.peb.modules
 
     def test_thread_owner_from_tid(self, proc32_64):
         thread = proc32_64.threads[0]
